/*     */ package com.google.code.yanf4j.core.impl;
/*     */ 
/*     */ import java.util.concurrent.CancellationException;
/*     */ import java.util.concurrent.CountDownLatch;
/*     */ import java.util.concurrent.ExecutionException;
/*     */ import java.util.concurrent.Future;
/*     */ import java.util.concurrent.TimeUnit;
/*     */ import java.util.concurrent.TimeoutException;
/*     */ import java.util.concurrent.locks.ReentrantLock;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class FutureLockImpl<R>
/*     */   implements Future<R>
/*     */ {
/*     */   private final ReentrantLock lock;
/*     */   private boolean isDone;
/*     */   private CountDownLatch latch;
/*     */   private boolean isCancelled;
/*     */   private Throwable failure;
/*     */   protected R result;
/*     */   
/*     */   public FutureLockImpl()
/*     */   {
/*  74 */     this(new ReentrantLock());
/*     */   }
/*     */   
/*     */   public FutureLockImpl(ReentrantLock lock)
/*     */   {
/*  79 */     this.lock = lock;
/*  80 */     this.latch = new CountDownLatch(1);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public R getResult()
/*     */   {
/*     */     try
/*     */     {
/*  91 */       this.lock.lock();
/*  92 */       return (R)this.result;
/*     */     }
/*     */     finally {
/*  95 */       this.lock.unlock();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void setResult(R result)
/*     */   {
/*     */     try
/*     */     {
/* 108 */       this.lock.lock();
/* 109 */       this.result = result;
/* 110 */       notifyHaveResult();
/*     */     }
/*     */     finally {
/* 113 */       this.lock.unlock();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public boolean cancel(boolean mayInterruptIfRunning)
/*     */   {
/*     */     try
/*     */     {
/* 123 */       this.lock.lock();
/* 124 */       this.isCancelled = true;
/* 125 */       notifyHaveResult();
/* 126 */       return true;
/*     */     }
/*     */     finally {
/* 129 */       this.lock.unlock();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public boolean isCancelled()
/*     */   {
/*     */     try
/*     */     {
/* 139 */       this.lock.lock();
/* 140 */       return this.isCancelled;
/*     */     }
/*     */     finally {
/* 143 */       this.lock.unlock();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public boolean isDone()
/*     */   {
/*     */     try
/*     */     {
/* 153 */       this.lock.lock();
/* 154 */       return this.isDone;
/*     */     }
/*     */     finally {
/* 157 */       this.lock.unlock();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public R get()
/*     */     throws InterruptedException, ExecutionException
/*     */   {
/* 166 */     this.latch.await();
/*     */     try
/*     */     {
/* 169 */       this.lock.lock();
/* 170 */       if (this.isCancelled) {
/* 171 */         throw new CancellationException();
/*     */       }
/* 173 */       if (this.failure != null) {
/* 174 */         throw new ExecutionException(this.failure);
/*     */       }
/*     */       
/* 177 */       return (R)this.result;
/*     */     }
/*     */     finally {
/* 180 */       this.lock.unlock();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public R get(long timeout, TimeUnit unit)
/*     */     throws InterruptedException, ExecutionException, TimeoutException
/*     */   {
/* 189 */     boolean isTimeOut = !this.latch.await(timeout, unit);
/*     */     try {
/* 191 */       this.lock.lock();
/* 192 */       if (!isTimeOut) {
/* 193 */         if (this.isCancelled) {
/* 194 */           throw new CancellationException();
/*     */         }
/* 196 */         if (this.failure != null) {
/* 197 */           throw new ExecutionException(this.failure);
/*     */         }
/*     */         
/* 200 */         return (R)this.result;
/*     */       }
/*     */       
/* 203 */       throw new TimeoutException();
/*     */     }
/*     */     finally
/*     */     {
/* 207 */       this.lock.unlock();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void failure(Throwable failure)
/*     */   {
/*     */     try
/*     */     {
/* 220 */       this.lock.lock();
/* 221 */       this.failure = failure;
/* 222 */       notifyHaveResult();
/*     */     }
/*     */     finally {
/* 225 */       this.lock.unlock();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void notifyHaveResult()
/*     */   {
/* 234 */     this.isDone = true;
/* 235 */     this.latch.countDown();
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\google\code\yanf4j\core\impl\FutureLockImpl.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */