/*     */ package com.netapp.asup.metamodel;
/*     */ 
/*     */ import java.util.ArrayList;
/*     */ import java.util.Collection;
/*     */ import java.util.HashMap;
/*     */ import java.util.HashSet;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Set;
/*     */ import prefuse.data.Edge;
/*     */ import prefuse.data.Node;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class NodeObject
/*     */ {
/*     */   private static final String TARGET_NAME = "target_name";
/*     */   private static final String SOURCE_NAME = "source_name";
/*     */   private static final String KEY_NAME = "key_name";
/*     */   public static final String DEFAULT_PARENT_NODE = "SYSTEM";
/*     */   public static final String DEFAULT_INSTANCE_KEY_NAME = "sys_id";
/*     */   private static final String NODE_ID = "nodeId";
/*     */   private static final String KEY_ATTRIBUTE = "key.attributes";
/*     */   private static final String MERGE_TYPE = "merge_type";
/*     */   private static final String IS_OPTIONS_SUPPORTED = "isOptionsSupported";
/*     */   private final Node node;
/*     */   private final MetaModel metaModel;
/*     */   private final MergeType mergeType;
/*  31 */   private Map<String, Relation> relationshipMap = new HashMap();
/*     */   
/*  33 */   private Set<String> parents = new HashSet();
/*  34 */   private Set<String> childs = new HashSet();
/*  35 */   private Map<String, String> virtualVsRealMap = new HashMap();
/*     */   private final CounterProperties counterProperties;
/*     */   
/*  38 */   public Map<String, String> getVirtualVsRealMap() { return this.virtualVsRealMap; }
/*     */   
/*     */   public Set<String> getParents()
/*     */   {
/*  42 */     return this.parents;
/*     */   }
/*     */   
/*     */   public Set<String> getChilds() {
/*  46 */     return this.childs;
/*     */   }
/*     */   
/*     */   public static enum RelationshipType {
/*  50 */     STANDARD, 
/*  51 */     MULTI_RELATION;
/*     */     
/*     */     private RelationshipType() {}
/*     */   }
/*     */   
/*  56 */   public static enum PCRType { PARENT, 
/*  57 */     CHILD;
/*     */     
/*     */     private PCRType() {}
/*     */   }
/*     */   
/*     */   public static enum CounterInstanceType {
/*  63 */     DEFAULT, 
/*  64 */     ATTRIBUTE, 
/*  65 */     INSTANCE_KEY, 
/*  66 */     NOT_AVAILABLE;
/*     */     
/*     */     private CounterInstanceType() {}
/*     */   }
/*     */   
/*  71 */   public Map<String, Relation> getRelationMap() { return this.relationshipMap; }
/*     */   
/*     */ 
/*     */   public Relation getRelationMap(String nodeName)
/*     */   {
/*  76 */     return (Relation)this.relationshipMap.get(nodeName);
/*     */   }
/*     */   
/*     */   private class CounterProperties
/*     */   {
/*     */     private static final String COUNTER_OBJECT_NAME = "counter.object.name";
/*     */     private static final String COUNTER_INSTANCE_KEY_ATTRIBUTE = "counter.instance.key.attribute";
/*     */     private static final String DEFAULT_COUNTER_INSTANCE_KEY = "default.counter.instance.key";
/*     */     private final String counterObject;
/*     */     private final String counterInstanceKeyAttribute;
/*     */     private final NodeObject.CounterInstanceType counterInstanceType;
/*     */     
/*     */     private CounterProperties()
/*     */     {
/*  90 */       this.counterObject = NodeObject.this.node.getString("counter.object.name");
/*  91 */       String counterInstanceKeyAttribute = null;
/*  92 */       if ((this.counterObject != null) && (!this.counterObject.isEmpty())) {
/*  93 */         counterInstanceKeyAttribute = NodeObject.this.node.getString("counter.instance.key.attribute");
/*  94 */         if ((counterInstanceKeyAttribute != null) && (!counterInstanceKeyAttribute.isEmpty())) {
/*  95 */           if (counterInstanceKeyAttribute.equals("instance_key")) {
/*  96 */             this.counterInstanceType = NodeObject.CounterInstanceType.INSTANCE_KEY;
/*     */           } else {
/*  98 */             this.counterInstanceType = NodeObject.CounterInstanceType.ATTRIBUTE;
/*     */           }
/*     */         } else {
/* 101 */           counterInstanceKeyAttribute = NodeObject.this.node.getString("default.counter.instance.key");
/* 102 */           if ((counterInstanceKeyAttribute != null) && (!counterInstanceKeyAttribute.isEmpty())) {
/* 103 */             this.counterInstanceType = NodeObject.CounterInstanceType.DEFAULT;
/*     */           } else {
/* 105 */             throw new RuntimeException("Invalid counter properties specified on node :" + NodeObject.this.node.getString("nodeId") + ". Counter has neither counter.instance.key.attribute nor default.counter.instance.key specified");
/*     */           }
/*     */         }
/*     */       } else {
/* 109 */         this.counterInstanceType = NodeObject.CounterInstanceType.NOT_AVAILABLE;
/*     */       }
/* 111 */       this.counterInstanceKeyAttribute = counterInstanceKeyAttribute;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   private List<String> attributeNames;
/*     */   
/*     */   public MetaModel getMetaModel()
/*     */   {
/* 120 */     return this.metaModel;
/*     */   }
/*     */   
/*     */ 
/*     */   private Set<String> populateNeighbourMap(Iterator<Node> nodeItr, Set<String> neighbors)
/*     */   {
/* 126 */     neighbors = new HashSet();
/* 127 */     while (nodeItr.hasNext()) {
/* 128 */       Node node = (Node)nodeItr.next();
/*     */       
/* 130 */       String name = node.getString("nodeId");
/* 131 */       neighbors.add(name);
/*     */     }
/*     */     
/* 134 */     return neighbors;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public void populateRelationMap(Node node)
/*     */   {
/* 142 */     Iterator<Edge> edgeIt = node.inEdges();
/*     */     
/* 144 */     Edge edge = null;
/* 145 */     NodeObject sourceObj = null;
/* 146 */     NodeObject targetObj = null;
/* 147 */     String sourceNodeStr = null;
/* 148 */     String targetNodeStr = null;
/*     */     
/* 150 */     while (edgeIt.hasNext())
/*     */     {
/* 152 */       edge = (Edge)edgeIt.next();
/* 153 */       sourceObj = getSourceNodeObject(edge);
/* 154 */       sourceNodeStr = edge.getSourceNode().getString("nodeId");
/* 155 */       updateRelationshipMap(edge, this.relationshipMap, sourceObj, sourceNodeStr, true);
/*     */     }
/*     */     
/*     */ 
/* 159 */     edgeIt = node.outEdges();
/* 160 */     while (edgeIt.hasNext())
/*     */     {
/* 162 */       edge = (Edge)edgeIt.next();
/* 163 */       targetObj = getTargetNodeObject(edge);
/* 164 */       targetNodeStr = edge.getTargetNode().getString("nodeId");
/* 165 */       updateRelationshipMap(edge, this.relationshipMap, targetObj, targetNodeStr, false);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public void updateNeighborNames(Iterator<String> it, boolean isParent)
/*     */   {
/* 174 */     Set<String> toAdd = new HashSet();
/* 175 */     Set<String> toRemove = new HashSet();
/* 176 */     List<Relation> multiRelation = null;
/* 177 */     String name = null;
/* 178 */     Relation relation = null;
/* 179 */     String virtualName = null;
/* 180 */     while (it.hasNext())
/*     */     {
/*     */ 
/* 183 */       name = (String)it.next();
/* 184 */       relation = (Relation)this.relationshipMap.get(name);
/* 185 */       if (relation.getType() == RelationshipType.MULTI_RELATION)
/*     */       {
/* 187 */         multiRelation = relation.getRelations();
/* 188 */         for (Relation r : multiRelation) {
/* 189 */           virtualName = r.getVirtualNodeName();
/* 190 */           toAdd.add(virtualName);
/* 191 */           this.virtualVsRealMap.put(virtualName, name);
/*     */         }
/*     */         
/* 194 */         toRemove.add(name);
/*     */       }
/*     */     }
/*     */     
/* 198 */     if (isParent) {
/* 199 */       if (toRemove.size() > 0) {
/* 200 */         this.parents.removeAll(toRemove);
/* 201 */         this.parents.addAll(toAdd);
/*     */       }
/*     */     }
/* 204 */     else if (toRemove.size() > 0) {
/* 205 */       this.childs.removeAll(toRemove);
/* 206 */       this.childs.addAll(toAdd);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public void updateVirtualPCRNames()
/*     */   {
/* 214 */     Iterator<String> parentIt = this.parents.iterator();
/* 215 */     Iterator<String> childIt = this.childs.iterator();
/* 216 */     updateNeighborNames(parentIt, true);
/* 217 */     updateNeighborNames(childIt, false);
/*     */   }
/*     */   
/*     */   public EdgeInfo getEdgeInfo(Edge edge, boolean isParent)
/*     */   {
/* 222 */     String keyName = null;
/* 223 */     String virtualNodeName = null;
/* 224 */     EdgeInfo edgeInfo = null;
/* 225 */     if ((edge.canGetString("key_name")) && (edge.getString("key_name").trim().length() > 0))
/*     */     {
/* 227 */       keyName = edge.getString("key_name"); }
/* 228 */     if (isParent) {
/* 229 */       if ((edge.canGetString("source_name")) && (edge.getString("source_name").trim().length() > 0))
/*     */       {
/* 231 */         virtualNodeName = edge.getString("source_name");
/*     */       }
/* 233 */       edgeInfo = new EdgeInfo(keyName, virtualNodeName, PCRType.PARENT);
/*     */     } else {
/* 235 */       if ((edge.canGetString("target_name")) && (edge.getString("target_name").trim().length() > 0))
/*     */       {
/* 237 */         virtualNodeName = edge.getString("target_name");
/*     */       }
/* 239 */       edgeInfo = new EdgeInfo(keyName, virtualNodeName, PCRType.CHILD);
/*     */     }
/*     */     
/* 242 */     return edgeInfo;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public void updateRelationshipMap(Edge edge, Map<String, Relation> relationshipMap, NodeObject nodeObject, String nodeName, boolean isParent)
/*     */   {
/* 250 */     EdgeInfo edgeInfo = null;
/* 251 */     Relation relation = (Relation)relationshipMap.get(nodeName);
/* 252 */     if (relation == null) {
/* 253 */       edgeInfo = getEdgeInfo(edge, isParent);
/* 254 */       relation = new StandardRelationImpl(RelationshipType.STANDARD, nodeName, nodeObject, edgeInfo.getPcrtype(), edgeInfo.getKeyName(), edgeInfo.getVirtualNodeName());
/*     */       
/*     */ 
/*     */ 
/* 258 */       relationshipMap.put(nodeName, relation);
/* 259 */     } else if (relation.getType() == RelationshipType.MULTI_RELATION)
/*     */     {
/* 261 */       edgeInfo = getEdgeInfo(edge, isParent);
/* 262 */       Relation newRelation = new StandardRelationImpl(RelationshipType.STANDARD, nodeName, nodeObject, edgeInfo.getPcrtype(), edgeInfo.getKeyName(), edgeInfo.getVirtualNodeName());
/*     */       
/*     */ 
/*     */ 
/*     */ 
/* 267 */       relation.addRelation(newRelation);
/*     */     }
/*     */     else
/*     */     {
/* 271 */       Relation firstRelation = relation;
/* 272 */       Relation newRelation = new MultiRelationImpl(RelationshipType.MULTI_RELATION);
/*     */       
/* 274 */       newRelation.addRelation(firstRelation);
/* 275 */       edgeInfo = getEdgeInfo(edge, isParent);
/* 276 */       Relation secondRelation = new StandardRelationImpl(RelationshipType.STANDARD, nodeName, nodeObject, edgeInfo.getPcrtype(), edgeInfo.getKeyName(), edgeInfo.getVirtualNodeName());
/*     */       
/*     */ 
/*     */ 
/*     */ 
/* 281 */       newRelation.addRelation(secondRelation);
/* 282 */       relationshipMap.put(nodeName, newRelation);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   private NodeObject getSourceNodeObject(Edge edge)
/*     */   {
/* 290 */     Node sourceNode = edge.getSourceNode();
/* 291 */     String sourceNodeStr = sourceNode.getString("nodeId");
/* 292 */     return getNodeObject(sourceNode, sourceNodeStr);
/*     */   }
/*     */   
/*     */   private NodeObject getTargetNodeObject(Edge edge)
/*     */   {
/* 297 */     Node targetNode = edge.getTargetNode();
/* 298 */     String targetNodeStr = targetNode.getString("nodeId");
/* 299 */     return getNodeObject(targetNode, targetNodeStr);
/*     */   }
/*     */   
/*     */ 
/*     */   private NodeObject getNodeObject(Node node, String nodeStr)
/*     */   {
/* 305 */     NodeObject nodeObj = this.metaModel.getNode(nodeStr);
/* 306 */     if (nodeObj == null) {
/* 307 */       nodeObj = new NodeObject(this.metaModel, node, false);
/* 308 */       this.metaModel.addToNodeMap(nodeObj);
/* 309 */       nodeObj.populateNeighboursMap();
/*     */     }
/*     */     
/* 312 */     return nodeObj;
/*     */   }
/*     */   
/*     */ 
/*     */   public NodeObject(MetaModel metaModel, Node node, boolean populateRelationMap)
/*     */   {
/* 318 */     this.metaModel = metaModel;
/* 319 */     this.node = node;
/* 320 */     this.relationshipMap = new HashMap();
/* 321 */     populateNeighboursMap();
/* 322 */     if (populateRelationMap) populateRelationMap(node);
/* 323 */     this.mergeType = MergeType.valueOf(node.getString("merge_type").toUpperCase());
/* 324 */     initAttributes();
/* 325 */     this.counterProperties = new CounterProperties(null);
/*     */   }
/*     */   
/*     */   private void populateNeighboursMap()
/*     */   {
/* 330 */     this.parents = populateParentMap();
/* 331 */     this.childs = populateChildMap();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   private Set<String> populateChildMap()
/*     */   {
/* 338 */     return populateNeighbourMap(this.node.outNeighbors(), this.childs);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   private Set<String> populateParentMap()
/*     */   {
/* 345 */     return populateNeighbourMap(this.node.inNeighbors(), this.parents);
/*     */   }
/*     */   
/*     */ 
/*     */   public Node getNode()
/*     */   {
/* 351 */     return this.node;
/*     */   }
/*     */   
/*     */   public String getName() {
/* 355 */     return getAttributeVal("nodeId");
/*     */   }
/*     */   
/*     */   private HashSet<NodeObject> updateNodeObjects(Relation relation, HashSet<NodeObject> nodeObjects) {
/* 359 */     for (Relation r : relation.getRelations()) {
/* 360 */       nodeObjects.add(r.getNodeObject());
/*     */     }
/* 362 */     return nodeObjects;
/*     */   }
/*     */   
/*     */   public Collection<NodeObject> getNodes(Iterator<String> it)
/*     */   {
/* 367 */     String name = null;
/* 368 */     Relation relation = null;
/* 369 */     String realName = null;
/* 370 */     HashSet<NodeObject> objects = new HashSet();
/* 371 */     HashSet<String> processedNames = new HashSet();
/* 372 */     while (it.hasNext())
/*     */     {
/* 374 */       name = (String)it.next();
/* 375 */       if (this.relationshipMap.containsKey(name)) {
/* 376 */         processedNames.add(name);
/* 377 */         relation = (Relation)this.relationshipMap.get(name);
/*     */         
/* 379 */         objects = updateNodeObjects(relation, objects);
/* 380 */       } else if (this.virtualVsRealMap.containsKey(name))
/*     */       {
/* 382 */         realName = (String)this.virtualVsRealMap.get(name);
/* 383 */         if (!processedNames.contains(realName)) {
/* 384 */           relation = (Relation)this.relationshipMap.get(realName);
/* 385 */           objects = updateNodeObjects(relation, objects);
/* 386 */           processedNames.add(realName);
/*     */         }
/*     */       }
/*     */     }
/*     */     
/* 391 */     return objects;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public Collection<NodeObject> getParentNodes()
/*     */   {
/* 398 */     Iterator<String> parentIt = this.parents.iterator();
/*     */     
/* 400 */     return getNodes(parentIt);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public Collection<NodeObject> getChildNodes()
/*     */   {
/* 407 */     Iterator<String> childIt = this.childs.iterator();
/* 408 */     return getNodes(childIt);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public List<NodeObject> getAllNodes()
/*     */   {
/* 415 */     List<NodeObject> allNodeList = new ArrayList();
/* 416 */     allNodeList.addAll(getParentNodes());
/* 417 */     allNodeList.addAll(getChildNodes());
/* 418 */     return allNodeList;
/*     */   }
/*     */   
/*     */   public Set<String> getAllNodeNames() {
/* 422 */     return this.metaModel.getAllNodeNames();
/*     */   }
/*     */   
/*     */   public NodeObject getNode(String nodeName) {
/* 426 */     return this.metaModel.getNode(nodeName);
/*     */   }
/*     */   
/*     */   public NodeObject getParentByName(String nodeName)
/*     */   {
/* 431 */     if ((!this.parents.contains(nodeName)) && (!this.virtualVsRealMap.containsValue(nodeName))) { return null;
/*     */     }
/* 433 */     return ((Relation)((Relation)this.relationshipMap.get(nodeName)).getRelations().get(0)).getNodeObject();
/*     */   }
/*     */   
/*     */ 
/*     */   public NodeObject getChildByName(String nodeName)
/*     */   {
/* 439 */     if ((!this.childs.contains(nodeName)) && (!this.virtualVsRealMap.containsValue(nodeName))) { return null;
/*     */     }
/* 441 */     return ((Relation)((Relation)this.relationshipMap.get(nodeName)).getRelations().get(0)).getNodeObject();
/*     */   }
/*     */   
/*     */ 
/*     */   public String getAttributeVal(String attributeName)
/*     */   {
/* 447 */     String value = this.node.getString(attributeName);
/* 448 */     return value;
/*     */   }
/*     */   
/*     */   public String getObjectName() {
/* 452 */     return this.node.getString("nodeId");
/*     */   }
/*     */   
/*     */   public String getKeyAttribute() {
/* 456 */     String value = this.node.getString("key.attributes");
/* 457 */     return value;
/*     */   }
/*     */   
/*     */   private void initAttributes() {
/* 461 */     Node node = getNode();
/* 462 */     int columnCount = node.getColumnCount();
/* 463 */     this.attributeNames = new ArrayList();
/* 464 */     for (int i = 0; i < columnCount; i++) {
/* 465 */       this.attributeNames.add(node.getColumnName(i));
/*     */     }
/*     */   }
/*     */   
/*     */   public int getColumnCount() {
/* 470 */     return this.node.getColumnCount();
/*     */   }
/*     */   
/*     */   public Object get(String columnName) {
/* 474 */     return this.node.get(columnName);
/*     */   }
/*     */   
/*     */   public Object getValueAt(int index) {
/* 478 */     return this.node.getValueAt(index);
/*     */   }
/*     */   
/*     */   public String getColumnName(int index) {
/* 482 */     return this.node.getColumnName(index);
/*     */   }
/*     */   
/*     */   public List<String> getAttributeNames() {
/* 486 */     return this.attributeNames;
/*     */   }
/*     */   
/*     */   public MergeType getMergeType() {
/* 490 */     return this.mergeType;
/*     */   }
/*     */   
/*     */   public boolean isOptionsSupported()
/*     */   {
/* 495 */     return this.node.getBoolean("isOptionsSupported");
/*     */   }
/*     */   
/*     */ 
/*     */   public String toString()
/*     */   {
/* 501 */     return "NodeName:" + getName();
/*     */   }
/*     */   
/*     */   public String getCounterObject() {
/* 505 */     return this.counterProperties.counterObject;
/*     */   }
/*     */   
/*     */   public String getCounterInstanceKeyAttribute() {
/* 509 */     return this.counterProperties.counterInstanceKeyAttribute;
/*     */   }
/*     */   
/*     */   public CounterInstanceType getCounterInstanceType() {
/* 513 */     return this.counterProperties.counterInstanceType;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\metamodel\NodeObject.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */