/*     */ package org.apache.avro.data;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.util.Iterator;
/*     */ import org.apache.avro.AvroRuntimeException;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.io.DatumReader;
/*     */ import org.apache.avro.io.DatumWriter;
/*     */ import org.apache.avro.io.Decoder;
/*     */ import org.apache.avro.io.DecoderFactory;
/*     */ import org.apache.avro.io.Encoder;
/*     */ import org.apache.avro.io.ResolvingDecoder;
/*     */ import org.codehaus.jackson.JsonNode;
/*     */ import org.codehaus.jackson.node.ArrayNode;
/*     */ import org.codehaus.jackson.node.BooleanNode;
/*     */ import org.codehaus.jackson.node.DoubleNode;
/*     */ import org.codehaus.jackson.node.JsonNodeFactory;
/*     */ import org.codehaus.jackson.node.LongNode;
/*     */ import org.codehaus.jackson.node.NullNode;
/*     */ import org.codehaus.jackson.node.ObjectNode;
/*     */ import org.codehaus.jackson.node.TextNode;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class Json
/*     */ {
/*     */   public static final Schema SCHEMA;
/*     */   
/*     */   static
/*     */   {
/*     */     try
/*     */     {
/*  50 */       SCHEMA = Schema.parse(Json.class.getResourceAsStream("/org/apache/avro/data/Json.avsc"));
/*     */     }
/*     */     catch (IOException e) {
/*  53 */       throw new AvroRuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   public static class Writer implements DatumWriter<JsonNode>
/*     */   {
/*     */     public void setSchema(Schema schema)
/*     */     {
/*  61 */       if (!Json.SCHEMA.equals(schema)) {
/*  62 */         throw new RuntimeException("Not the Json schema: " + schema);
/*     */       }
/*     */     }
/*     */     
/*     */     public void write(JsonNode datum, Encoder out) throws IOException {
/*  67 */       Json.write(datum, out);
/*     */     }
/*     */   }
/*     */   
/*     */   public static class Reader implements DatumReader<JsonNode>
/*     */   {
/*     */     private Schema written;
/*     */     private ResolvingDecoder resolver;
/*     */     
/*     */     public void setSchema(Schema schema) {
/*  77 */       this.written = (Json.SCHEMA.equals(this.written) ? null : schema);
/*     */     }
/*     */     
/*     */     public JsonNode read(JsonNode reuse, Decoder in) throws IOException
/*     */     {
/*  82 */       if (this.written == null) {
/*  83 */         return Json.read(in);
/*     */       }
/*     */       
/*  86 */       if (this.resolver == null)
/*  87 */         this.resolver = DecoderFactory.get().resolvingDecoder(this.written, Json.SCHEMA, null);
/*  88 */       this.resolver.configure(in);
/*  89 */       JsonNode result = Json.read(this.resolver);
/*  90 */       this.resolver.drain();
/*  91 */       return result;
/*     */     }
/*     */   }
/*     */   
/*     */   private static enum JsonType {
/*  96 */     LONG,  DOUBLE,  STRING,  BOOLEAN,  NULL,  ARRAY,  OBJECT;
/*     */     
/*     */     private JsonType() {} }
/*     */   
/* 100 */   public static void write(JsonNode node, Encoder out) throws IOException { switch (node.asToken()) {
/*     */     case VALUE_NUMBER_INT: 
/* 102 */       out.writeIndex(JsonType.LONG.ordinal());
/* 103 */       out.writeLong(node.getLongValue());
/* 104 */       break;
/*     */     case VALUE_NUMBER_FLOAT: 
/* 106 */       out.writeIndex(JsonType.DOUBLE.ordinal());
/* 107 */       out.writeDouble(node.getDoubleValue());
/* 108 */       break;
/*     */     case VALUE_STRING: 
/* 110 */       out.writeIndex(JsonType.STRING.ordinal());
/* 111 */       out.writeString(node.getTextValue());
/* 112 */       break;
/*     */     case VALUE_TRUE: 
/* 114 */       out.writeIndex(JsonType.BOOLEAN.ordinal());
/* 115 */       out.writeBoolean(true);
/* 116 */       break;
/*     */     case VALUE_FALSE: 
/* 118 */       out.writeIndex(JsonType.BOOLEAN.ordinal());
/* 119 */       out.writeBoolean(false);
/* 120 */       break;
/*     */     case VALUE_NULL: 
/* 122 */       out.writeIndex(JsonType.NULL.ordinal());
/* 123 */       out.writeNull();
/* 124 */       break;
/*     */     case START_ARRAY: 
/* 126 */       out.writeIndex(JsonType.ARRAY.ordinal());
/* 127 */       out.writeArrayStart();
/* 128 */       out.setItemCount(node.size());
/* 129 */       for (JsonNode element : node) {
/* 130 */         out.startItem();
/* 131 */         write(element, out);
/*     */       }
/* 133 */       out.writeArrayEnd();
/* 134 */       break;
/*     */     case START_OBJECT: 
/* 136 */       out.writeIndex(JsonType.OBJECT.ordinal());
/* 137 */       out.writeMapStart();
/* 138 */       out.setItemCount(node.size());
/* 139 */       Iterator<String> i = node.getFieldNames();
/* 140 */       while (i.hasNext()) {
/* 141 */         out.startItem();
/* 142 */         String name = (String)i.next();
/* 143 */         out.writeString(name);
/* 144 */         write(node.get(name), out);
/*     */       }
/* 146 */       out.writeMapEnd();
/* 147 */       break;
/*     */     default: 
/* 149 */       throw new AvroRuntimeException(node.asToken() + " unexpected: " + node);
/*     */     }
/*     */   }
/*     */   
/*     */   public static JsonNode read(Decoder in) throws IOException
/*     */   {
/* 155 */     switch (JsonType.values()[in.readIndex()]) {
/*     */     case LONG: 
/* 157 */       return new LongNode(in.readLong());
/*     */     case DOUBLE: 
/* 159 */       return new DoubleNode(in.readDouble());
/*     */     case STRING: 
/* 161 */       return new TextNode(in.readString());
/*     */     case BOOLEAN: 
/* 163 */       return in.readBoolean() ? BooleanNode.TRUE : BooleanNode.FALSE;
/*     */     case NULL: 
/* 165 */       in.readNull();
/* 166 */       return NullNode.getInstance();
/*     */     case ARRAY: 
/* 168 */       ArrayNode array = JsonNodeFactory.instance.arrayNode();
/* 169 */       for (long l = in.readArrayStart(); l > 0L; l = in.arrayNext())
/* 170 */         for (long i = 0L; i < l; i += 1L)
/* 171 */           array.add(read(in));
/* 172 */       return array;
/*     */     case OBJECT: 
/* 174 */       ObjectNode object = JsonNodeFactory.instance.objectNode();
/* 175 */       for (long l = in.readMapStart(); l > 0L; l = in.mapNext())
/* 176 */         for (long i = 0L; i < l; i += 1L)
/* 177 */           object.put(in.readString(), read(in));
/* 178 */       return object;
/*     */     }
/* 180 */     throw new AvroRuntimeException("Unexpected Json node type");
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\data\Json.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */