/*     */ package org.apache.avro.file;
/*     */ 
/*     */ import java.io.Closeable;
/*     */ import java.io.IOException;
/*     */ import java.io.UnsupportedEncodingException;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.util.Arrays;
/*     */ import java.util.HashMap;
/*     */ import java.util.Iterator;
/*     */ import java.util.Map;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.io.BinaryDecoder;
/*     */ import org.apache.avro.io.DatumReader;
/*     */ import org.apache.avro.io.DecoderFactory;
/*     */ import org.apache.avro.util.Utf8;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class DataFileReader12<D>
/*     */   implements FileReader<D>, Closeable
/*     */ {
/*     */   private static final byte VERSION = 0;
/*  37 */   static final byte[] MAGIC = { 79, 98, 106, 0 };
/*     */   
/*     */   private static final long FOOTER_BLOCK = -1L;
/*     */   
/*     */   private static final int SYNC_SIZE = 16;
/*     */   
/*     */   private static final int SYNC_INTERVAL = 16000;
/*     */   
/*     */   private static final String SCHEMA = "schema";
/*     */   
/*     */   private static final String SYNC = "sync";
/*     */   private static final String COUNT = "count";
/*     */   private static final String CODEC = "codec";
/*     */   private static final String NULL_CODEC = "null";
/*     */   private Schema schema;
/*     */   private DatumReader<D> reader;
/*     */   private DataFileReader.SeekableInputStream in;
/*     */   private BinaryDecoder vin;
/*  55 */   private Map<String, byte[]> meta = new HashMap();
/*     */   
/*     */   private long count;
/*     */   private long blockCount;
/*     */   private long blockStart;
/*  60 */   private byte[] sync = new byte[16];
/*  61 */   private byte[] syncBuffer = new byte[16];
/*     */   private D peek;
/*     */   
/*     */   public DataFileReader12(SeekableInput sin, DatumReader<D> reader) throws IOException
/*     */   {
/*  66 */     this.in = new DataFileReader.SeekableInputStream(sin);
/*     */     
/*  68 */     byte[] magic = new byte[4];
/*  69 */     this.in.read(magic);
/*  70 */     if (!Arrays.equals(MAGIC, magic)) {
/*  71 */       throw new IOException("Not a data file.");
/*     */     }
/*  73 */     long length = this.in.length();
/*  74 */     this.in.seek(length - 4L);
/*  75 */     int footerSize = (this.in.read() << 24) + (this.in.read() << 16) + (this.in.read() << 8) + this.in.read();
/*  76 */     seek(length - footerSize);
/*  77 */     long l = this.vin.readMapStart();
/*  78 */     if (l > 0L) {
/*     */       do {
/*  80 */         for (long i = 0L; i < l; i += 1L) {
/*  81 */           String key = this.vin.readString(null).toString();
/*  82 */           ByteBuffer value = this.vin.readBytes(null);
/*  83 */           byte[] bb = new byte[value.remaining()];
/*  84 */           value.get(bb);
/*  85 */           this.meta.put(key, bb);
/*     */         }
/*  87 */       } while ((l = this.vin.mapNext()) != 0L);
/*     */     }
/*     */     
/*  90 */     this.sync = getMeta("sync");
/*  91 */     this.count = getMetaLong("count");
/*  92 */     String codec = getMetaString("codec");
/*  93 */     if ((codec != null) && (!codec.equals("null"))) {
/*  94 */       throw new IOException("Unknown codec: " + codec);
/*     */     }
/*  96 */     this.schema = Schema.parse(getMetaString("schema"));
/*  97 */     this.reader = reader;
/*     */     
/*  99 */     reader.setSchema(this.schema);
/*     */     
/* 101 */     seek(MAGIC.length);
/*     */   }
/*     */   
/*     */   public synchronized byte[] getMeta(String key)
/*     */   {
/* 106 */     return (byte[])this.meta.get(key);
/*     */   }
/*     */   
/*     */   public synchronized String getMetaString(String key) {
/* 110 */     byte[] value = getMeta(key);
/* 111 */     if (value == null) {
/* 112 */       return null;
/*     */     }
/*     */     try {
/* 115 */       return new String(value, "UTF-8");
/*     */     } catch (UnsupportedEncodingException e) {
/* 117 */       throw new RuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   public synchronized long getMetaLong(String key) {
/* 122 */     return Long.parseLong(getMetaString(key));
/*     */   }
/*     */   
/*     */   public Schema getSchema()
/*     */   {
/* 127 */     return this.schema;
/*     */   }
/*     */   
/*     */ 
/* 131 */   public Iterator<D> iterator() { return this; }
/*     */   
/* 133 */   public boolean hasNext() { if ((this.peek != null) || (this.blockCount != 0L))
/* 134 */       return true;
/* 135 */     this.peek = next();
/* 136 */     return this.peek != null;
/*     */   }
/*     */   
/* 139 */   public D next() { if (this.peek != null) {
/* 140 */       D result = this.peek;
/* 141 */       this.peek = null;
/* 142 */       return result;
/*     */     }
/*     */     try {
/* 145 */       return (D)next(null);
/*     */     } catch (IOException e) {
/* 147 */       throw new RuntimeException(e);
/*     */     } }
/*     */   
/* 150 */   public void remove() { throw new UnsupportedOperationException(); }
/*     */   
/*     */   public synchronized D next(D reuse)
/*     */     throws IOException
/*     */   {
/* 155 */     while (this.blockCount == 0L)
/*     */     {
/* 157 */       if (this.in.tell() == this.in.length()) {
/* 158 */         return null;
/*     */       }
/* 160 */       skipSync();
/*     */       
/* 162 */       this.blockCount = this.vin.readLong();
/*     */       
/* 164 */       if (this.blockCount == -1L) {
/* 165 */         seek(this.vin.readLong() + this.in.tell());
/*     */       }
/*     */     }
/* 168 */     this.blockCount -= 1L;
/* 169 */     return (D)this.reader.read(reuse, this.vin);
/*     */   }
/*     */   
/*     */   private void skipSync() throws IOException {
/* 173 */     this.vin.readFixed(this.syncBuffer);
/* 174 */     if (!Arrays.equals(this.syncBuffer, this.sync)) {
/* 175 */       throw new IOException("Invalid sync!");
/*     */     }
/*     */   }
/*     */   
/*     */   public synchronized void seek(long position) throws IOException
/*     */   {
/* 181 */     this.in.seek(position);
/* 182 */     this.blockCount = 0L;
/* 183 */     this.blockStart = position;
/* 184 */     this.vin = DecoderFactory.get().binaryDecoder(this.in, this.vin);
/*     */   }
/*     */   
/*     */   public synchronized void sync(long position)
/*     */     throws IOException
/*     */   {
/* 190 */     if (this.in.tell() + 16L >= this.in.length()) {
/* 191 */       seek(this.in.length());
/* 192 */       return;
/*     */     }
/* 194 */     this.in.seek(position);
/* 195 */     this.vin.readFixed(this.syncBuffer);
/* 196 */     for (int i = 0; this.in.tell() < this.in.length(); i++) {
/* 197 */       for (int j = 0; 
/* 198 */           j < this.sync.length; j++) {
/* 199 */         if (this.sync[j] != this.syncBuffer[((i + j) % this.sync.length)])
/*     */           break;
/*     */       }
/* 202 */       if (j == this.sync.length) {
/* 203 */         seek(this.in.tell() - 16L);
/* 204 */         return;
/*     */       }
/* 206 */       this.syncBuffer[(i % this.sync.length)] = ((byte)this.in.read());
/*     */     }
/* 208 */     seek(this.in.length());
/*     */   }
/*     */   
/*     */   public boolean pastSync(long position)
/*     */     throws IOException
/*     */   {
/* 214 */     return (this.blockStart >= position + 16L) || (this.blockStart >= this.in.length());
/*     */   }
/*     */   
/*     */   public long tell() throws IOException
/*     */   {
/* 219 */     return this.in.tell();
/*     */   }
/*     */   
/*     */   public synchronized void close() throws IOException
/*     */   {
/* 224 */     this.in.close();
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\file\DataFileReader12.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */