/*     */ package org.apache.avro.generic;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.util.Collection;
/*     */ import java.util.HashMap;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import org.apache.avro.AvroRuntimeException;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.Schema.Field;
/*     */ import org.apache.avro.io.DatumReader;
/*     */ import org.apache.avro.io.Decoder;
/*     */ import org.apache.avro.io.DecoderFactory;
/*     */ import org.apache.avro.io.ResolvingDecoder;
/*     */ import org.apache.avro.util.Utf8;
/*     */ import org.apache.avro.util.WeakIdentityHashMap;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class GenericDatumReader<D>
/*     */   implements DatumReader<D>
/*     */ {
/*     */   private final GenericData data;
/*     */   private Schema actual;
/*     */   private Schema expected;
/*  42 */   private ResolvingDecoder creatorResolver = null;
/*     */   private final Thread creator;
/*     */   
/*     */   public GenericDatumReader() {
/*  46 */     this(null, null, GenericData.get());
/*     */   }
/*     */   
/*     */   public GenericDatumReader(Schema schema)
/*     */   {
/*  51 */     this(schema, schema, GenericData.get());
/*     */   }
/*     */   
/*     */   public GenericDatumReader(Schema writer, Schema reader)
/*     */   {
/*  56 */     this(writer, reader, GenericData.get());
/*     */   }
/*     */   
/*     */   protected GenericDatumReader(Schema writer, Schema reader, GenericData data) {
/*  60 */     this.actual = writer;
/*  61 */     this.expected = reader;
/*  62 */     this.data = data;
/*  63 */     this.creator = Thread.currentThread();
/*     */   }
/*     */   
/*     */   public GenericData getData() {
/*  67 */     return this.data;
/*     */   }
/*     */   
/*  70 */   public Schema getSchema() { return this.actual; }
/*     */   
/*     */   public void setSchema(Schema writer)
/*     */   {
/*  74 */     this.actual = writer;
/*  75 */     if (this.expected == null) {
/*  76 */       this.expected = this.actual;
/*     */     }
/*  78 */     this.creatorResolver = null;
/*     */   }
/*     */   
/*     */   public Schema getExpected() {
/*  82 */     return this.expected;
/*     */   }
/*     */   
/*     */   public void setExpected(Schema reader) {
/*  86 */     this.expected = reader;
/*  87 */     this.creatorResolver = null;
/*     */   }
/*     */   
/*     */ 
/*  91 */   private static final ThreadLocal<Map<Schema, Map<Schema, ResolvingDecoder>>> RESOLVER_CACHE = new ThreadLocal()
/*     */   {
/*     */     protected Map<Schema, Map<Schema, ResolvingDecoder>> initialValue() {
/*  94 */       return new WeakIdentityHashMap();
/*     */     }
/*     */   };
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final ResolvingDecoder getResolver(Schema actual, Schema expected)
/*     */     throws IOException
/*     */   {
/* 105 */     Thread currThread = Thread.currentThread();
/*     */     
/* 107 */     if ((currThread == this.creator) && (this.creatorResolver != null)) {
/* 108 */       return this.creatorResolver;
/*     */     }
/*     */     
/* 111 */     Map<Schema, ResolvingDecoder> cache = (Map)((Map)RESOLVER_CACHE.get()).get(actual);
/* 112 */     if (cache == null) {
/* 113 */       cache = new WeakIdentityHashMap();
/* 114 */       ((Map)RESOLVER_CACHE.get()).put(actual, cache);
/*     */     }
/* 116 */     ResolvingDecoder resolver = (ResolvingDecoder)cache.get(expected);
/* 117 */     if (resolver == null) {
/* 118 */       resolver = DecoderFactory.get().resolvingDecoder(Schema.applyAliases(actual, expected), expected, null);
/*     */       
/* 120 */       cache.put(expected, resolver);
/*     */     }
/*     */     
/* 123 */     if (currThread == this.creator) {
/* 124 */       this.creatorResolver = resolver;
/*     */     }
/*     */     
/* 127 */     return resolver;
/*     */   }
/*     */   
/*     */   public D read(D reuse, Decoder in)
/*     */     throws IOException
/*     */   {
/* 133 */     ResolvingDecoder resolver = getResolver(this.actual, this.expected);
/* 134 */     resolver.configure(in);
/* 135 */     D result = read(reuse, this.expected, resolver);
/* 136 */     resolver.drain();
/* 137 */     return result;
/*     */   }
/*     */   
/*     */   protected Object read(Object old, Schema expected, ResolvingDecoder in)
/*     */     throws IOException
/*     */   {
/* 143 */     switch (expected.getType()) {
/* 144 */     case RECORD:  return readRecord(old, expected, in);
/* 145 */     case ENUM:  return readEnum(expected, in);
/* 146 */     case ARRAY:  return readArray(old, expected, in);
/* 147 */     case MAP:  return readMap(old, expected, in);
/* 148 */     case UNION:  return read(old, (Schema)expected.getTypes().get(in.readIndex()), in);
/* 149 */     case FIXED:  return readFixed(old, expected, in);
/* 150 */     case STRING:  return readString(old, expected, in);
/* 151 */     case BYTES:  return readBytes(old, expected, in);
/* 152 */     case INT:  return readInt(old, expected, in);
/* 153 */     case LONG:  return Long.valueOf(in.readLong());
/* 154 */     case FLOAT:  return Float.valueOf(in.readFloat());
/* 155 */     case DOUBLE:  return Double.valueOf(in.readDouble());
/* 156 */     case BOOLEAN:  return Boolean.valueOf(in.readBoolean());
/* 157 */     case NULL:  in.readNull();return null; }
/* 158 */     throw new AvroRuntimeException("Unknown type: " + expected);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected Object readRecord(Object old, Schema expected, ResolvingDecoder in)
/*     */     throws IOException
/*     */   {
/* 166 */     Object r = this.data.newRecord(old, expected);
/* 167 */     Object state = this.data.getRecordState(r, expected);
/*     */     
/* 169 */     for (Schema.Field f : in.readFieldOrder()) {
/* 170 */       int pos = f.pos();
/* 171 */       String name = f.name();
/* 172 */       Object oldDatum = old != null ? this.data.getField(r, name, pos, state) : null;
/* 173 */       this.data.setField(r, name, pos, read(oldDatum, f.schema(), in), state);
/*     */     }
/*     */     
/* 176 */     return r;
/*     */   }
/*     */   
/*     */   protected Object readEnum(Schema expected, Decoder in)
/*     */     throws IOException
/*     */   {
/* 182 */     return createEnum((String)expected.getEnumSymbols().get(in.readEnum()), expected);
/*     */   }
/*     */   
/*     */ 
/*     */   protected Object createEnum(String symbol, Schema schema)
/*     */   {
/* 188 */     return new GenericData.EnumSymbol(schema, symbol);
/*     */   }
/*     */   
/*     */ 
/*     */   protected Object readArray(Object old, Schema expected, ResolvingDecoder in)
/*     */     throws IOException
/*     */   {
/* 195 */     Schema expectedType = expected.getElementType();
/* 196 */     long l = in.readArrayStart();
/* 197 */     long base = 0L;
/* 198 */     if (l > 0L) {
/* 199 */       Object array = newArray(old, (int)l, expected);
/*     */       do {
/* 201 */         for (long i = 0L; i < l; i += 1L) {
/* 202 */           addToArray(array, base + i, read(peekArray(array), expectedType, in));
/*     */         }
/* 204 */         base += l;
/* 205 */       } while ((l = in.arrayNext()) > 0L);
/* 206 */       return array;
/*     */     }
/* 208 */     return newArray(old, 0, expected);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Object peekArray(Object array)
/*     */   {
/* 217 */     return (array instanceof GenericArray) ? ((GenericArray)array).peek() : null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void addToArray(Object array, long pos, Object e)
/*     */   {
/* 226 */     ((Collection)array).add(e);
/*     */   }
/*     */   
/*     */ 
/*     */   protected Object readMap(Object old, Schema expected, ResolvingDecoder in)
/*     */     throws IOException
/*     */   {
/* 233 */     Schema eValue = expected.getValueType();
/* 234 */     long l = in.readMapStart();
/* 235 */     Object map = newMap(old, (int)l);
/* 236 */     if (l > 0L) {
/*     */       do {
/* 238 */         for (int i = 0; i < l; i++) {
/* 239 */           addToMap(map, readString(null, expected, in), read(null, eValue, in));
/*     */         }
/* 241 */       } while ((l = in.mapNext()) > 0L);
/*     */     }
/* 243 */     return map;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected void addToMap(Object map, Object key, Object value)
/*     */   {
/* 250 */     ((Map)map).put(key, value);
/*     */   }
/*     */   
/*     */ 
/*     */   protected Object readFixed(Object old, Schema expected, Decoder in)
/*     */     throws IOException
/*     */   {
/* 257 */     GenericFixed fixed = (GenericFixed)this.data.createFixed(old, expected);
/* 258 */     in.readFixed(fixed.bytes(), 0, expected.getFixedSize());
/* 259 */     return fixed;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @Deprecated
/*     */   protected Object createFixed(Object old, Schema schema)
/*     */   {
/* 270 */     return this.data.createFixed(old, schema);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @Deprecated
/*     */   protected Object createFixed(Object old, byte[] bytes, Schema schema)
/*     */   {
/* 281 */     return this.data.createFixed(old, bytes, schema);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @Deprecated
/*     */   protected Object newRecord(Object old, Schema schema)
/*     */   {
/* 296 */     return this.data.newRecord(old, schema);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Object newArray(Object old, int size, Schema schema)
/*     */   {
/* 304 */     if ((old instanceof Collection)) {
/* 305 */       ((Collection)old).clear();
/* 306 */       return old; }
/* 307 */     return new GenericData.Array(size, schema);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Object newMap(Object old, int size)
/*     */   {
/* 315 */     if ((old instanceof Map)) {
/* 316 */       ((Map)old).clear();
/* 317 */       return old; }
/* 318 */     return new HashMap(size);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected Object readString(Object old, Schema expected, Decoder in)
/*     */     throws IOException
/*     */   {
/* 326 */     if ("String".equals(expected.getProp("avro.java.string"))) {
/* 327 */       return in.readString();
/*     */     }
/* 329 */     return readString(old, in);
/*     */   }
/*     */   
/*     */ 
/*     */   protected Object readString(Object old, Decoder in)
/*     */     throws IOException
/*     */   {
/* 336 */     return in.readString((old instanceof Utf8) ? (Utf8)old : null);
/*     */   }
/*     */   
/*     */ 
/*     */   protected Object createString(String value)
/*     */   {
/* 342 */     return new Utf8(value);
/*     */   }
/*     */   
/*     */ 
/*     */   protected Object readBytes(Object old, Schema s, Decoder in)
/*     */     throws IOException
/*     */   {
/* 349 */     return readBytes(old, in);
/*     */   }
/*     */   
/*     */ 
/*     */   protected Object readBytes(Object old, Decoder in)
/*     */     throws IOException
/*     */   {
/* 356 */     return in.readBytes((old instanceof ByteBuffer) ? (ByteBuffer)old : null);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected Object readInt(Object old, Schema expected, Decoder in)
/*     */     throws IOException
/*     */   {
/* 364 */     return Integer.valueOf(in.readInt());
/*     */   }
/*     */   
/*     */ 
/*     */   protected Object createBytes(byte[] value)
/*     */   {
/* 370 */     return ByteBuffer.wrap(value);
/*     */   }
/*     */   
/*     */   public static void skip(Schema schema, Decoder in) throws IOException {
/* 374 */     switch (schema.getType()) {
/*     */     case RECORD: 
/* 376 */       for (Schema.Field field : schema.getFields())
/* 377 */         skip(field.schema(), in);
/* 378 */       break;
/*     */     case ENUM: 
/* 380 */       in.readInt();
/* 381 */       break;
/*     */     case ARRAY: 
/* 383 */       Schema elementType = schema.getElementType();
/* 384 */       for (long l = in.skipArray(); l > 0L; l = in.skipArray()) {
/* 385 */         for (long i = 0L; i < l; i += 1L) {
/* 386 */           skip(elementType, in);
/*     */         }
/*     */       }
/* 389 */       break;
/*     */     case MAP: 
/* 391 */       Schema value = schema.getValueType();
/* 392 */       for (long l = in.skipMap(); l > 0L; l = in.skipMap()) {
/* 393 */         for (long i = 0L; i < l; i += 1L) {
/* 394 */           in.skipString();
/* 395 */           skip(value, in);
/*     */         }
/*     */       }
/* 398 */       break;
/*     */     case UNION: 
/* 400 */       skip((Schema)schema.getTypes().get(in.readIndex()), in);
/* 401 */       break;
/*     */     case FIXED: 
/* 403 */       in.skipFixed(schema.getFixedSize());
/* 404 */       break;
/*     */     case STRING: 
/* 406 */       in.skipString();
/* 407 */       break;
/*     */     case BYTES: 
/* 409 */       in.skipBytes();
/* 410 */       break;
/* 411 */     case INT:  in.readInt(); break;
/* 412 */     case LONG:  in.readLong(); break;
/* 413 */     case FLOAT:  in.readFloat(); break;
/* 414 */     case DOUBLE:  in.readDouble(); break;
/* 415 */     case BOOLEAN:  in.readBoolean(); break;
/*     */     case NULL: 
/*     */       break; default:  throw new RuntimeException("Unknown type: " + schema);
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\generic\GenericDatumReader.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */