/*     */ package org.codehaus.jackson.impl;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.io.Reader;
/*     */ import java.io.Writer;
/*     */ import org.codehaus.jackson.JsonParseException;
/*     */ import org.codehaus.jackson.JsonParser.Feature;
/*     */ import org.codehaus.jackson.io.IOContext;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class ReaderBasedParserBase
/*     */   extends JsonNumericParserBase
/*     */ {
/*     */   protected Reader _reader;
/*     */   protected char[] _inputBuffer;
/*     */   
/*     */   protected ReaderBasedParserBase(IOContext ctxt, int features, Reader r)
/*     */   {
/*  53 */     super(ctxt, features);
/*  54 */     this._reader = r;
/*  55 */     this._inputBuffer = ctxt.allocTokenBuffer();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int releaseBuffered(Writer w)
/*     */     throws IOException
/*     */   {
/*  67 */     int count = this._inputEnd - this._inputPtr;
/*  68 */     if (count < 1) {
/*  69 */       return 0;
/*     */     }
/*     */     
/*  72 */     int origPtr = this._inputPtr;
/*  73 */     w.write(this._inputBuffer, origPtr, count);
/*  74 */     return count;
/*     */   }
/*     */   
/*     */   public Object getInputSource()
/*     */   {
/*  79 */     return this._reader;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final boolean loadMore()
/*     */     throws IOException
/*     */   {
/*  91 */     this._currInputProcessed += this._inputEnd;
/*  92 */     this._currInputRowStart -= this._inputEnd;
/*     */     
/*  94 */     if (this._reader != null) {
/*  95 */       int count = this._reader.read(this._inputBuffer, 0, this._inputBuffer.length);
/*  96 */       if (count > 0) {
/*  97 */         this._inputPtr = 0;
/*  98 */         this._inputEnd = count;
/*  99 */         return true;
/*     */       }
/*     */       
/* 102 */       _closeInput();
/*     */       
/* 104 */       if (count == 0) {
/* 105 */         throw new IOException("Reader returned 0 characters when trying to read " + this._inputEnd);
/*     */       }
/*     */     }
/* 108 */     return false;
/*     */   }
/*     */   
/*     */   protected char getNextChar(String eofMsg)
/*     */     throws IOException, JsonParseException
/*     */   {
/* 114 */     if ((this._inputPtr >= this._inputEnd) && 
/* 115 */       (!loadMore())) {
/* 116 */       _reportInvalidEOF(eofMsg);
/*     */     }
/*     */     
/* 119 */     return this._inputBuffer[(this._inputPtr++)];
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void _closeInput()
/*     */     throws IOException
/*     */   {
/* 132 */     if (this._reader != null) {
/* 133 */       if ((this._ioContext.isResourceManaged()) || (isEnabled(JsonParser.Feature.AUTO_CLOSE_SOURCE))) {
/* 134 */         this._reader.close();
/*     */       }
/* 136 */       this._reader = null;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void _releaseBuffers()
/*     */     throws IOException
/*     */   {
/* 150 */     super._releaseBuffers();
/* 151 */     char[] buf = this._inputBuffer;
/* 152 */     if (buf != null) {
/* 153 */       this._inputBuffer = null;
/* 154 */       this._ioContext.releaseTokenBuffer(buf);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final boolean _matchToken(String matchStr, int i)
/*     */     throws IOException, JsonParseException
/*     */   {
/* 172 */     int len = matchStr.length();
/*     */     do
/*     */     {
/* 175 */       if ((this._inputPtr >= this._inputEnd) && 
/* 176 */         (!loadMore())) {
/* 177 */         _reportInvalidEOFInValue();
/*     */       }
/*     */       
/* 180 */       if (this._inputBuffer[this._inputPtr] != matchStr.charAt(i)) {
/* 181 */         _reportInvalidToken(matchStr.substring(0, i), "'null', 'true', 'false' or NaN");
/*     */       }
/* 183 */       this._inputPtr += 1;
/* 184 */       i++; } while (i < len);
/*     */     
/*     */ 
/* 187 */     if ((this._inputPtr >= this._inputEnd) && 
/* 188 */       (!loadMore())) {
/* 189 */       return true;
/*     */     }
/*     */     
/* 192 */     char c = this._inputBuffer[this._inputPtr];
/*     */     
/* 194 */     if (Character.isJavaIdentifierPart(c)) {
/* 195 */       this._inputPtr += 1;
/* 196 */       _reportInvalidToken(matchStr.substring(0, i), "'null', 'true', 'false' or NaN");
/*     */     }
/* 198 */     return true;
/*     */   }
/*     */   
/*     */   protected void _reportInvalidToken(String matchedPart, String msg)
/*     */     throws IOException, JsonParseException
/*     */   {
/* 204 */     StringBuilder sb = new StringBuilder(matchedPart);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 210 */     while ((this._inputPtr < this._inputEnd) || 
/* 211 */       (loadMore()))
/*     */     {
/*     */ 
/*     */ 
/* 215 */       char c = this._inputBuffer[this._inputPtr];
/* 216 */       if (!Character.isJavaIdentifierPart(c)) {
/*     */         break;
/*     */       }
/* 219 */       this._inputPtr += 1;
/* 220 */       sb.append(c);
/*     */     }
/* 222 */     _reportError("Unrecognized token '" + sb.toString() + "': was expecting ");
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\impl\ReaderBasedParserBase.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */