/*     */ package org.codehaus.jackson.map;
/*     */ 
/*     */ import java.io.Closeable;
/*     */ import java.io.File;
/*     */ import java.io.IOException;
/*     */ import java.io.OutputStream;
/*     */ import java.io.Writer;
/*     */ import org.codehaus.jackson.FormatSchema;
/*     */ import org.codehaus.jackson.JsonEncoding;
/*     */ import org.codehaus.jackson.JsonFactory;
/*     */ import org.codehaus.jackson.JsonGenerationException;
/*     */ import org.codehaus.jackson.JsonGenerator;
/*     */ import org.codehaus.jackson.PrettyPrinter;
/*     */ import org.codehaus.jackson.Version;
/*     */ import org.codehaus.jackson.Versioned;
/*     */ import org.codehaus.jackson.io.SegmentedStringWriter;
/*     */ import org.codehaus.jackson.map.ser.FilterProvider;
/*     */ import org.codehaus.jackson.map.type.TypeFactory;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ import org.codehaus.jackson.type.TypeReference;
/*     */ import org.codehaus.jackson.util.ByteArrayBuilder;
/*     */ import org.codehaus.jackson.util.DefaultPrettyPrinter;
/*     */ import org.codehaus.jackson.util.MinimalPrettyPrinter;
/*     */ import org.codehaus.jackson.util.VersionUtil;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ObjectWriter
/*     */   implements Versioned
/*     */ {
/*  33 */   protected static final PrettyPrinter NULL_PRETTY_PRINTER = new MinimalPrettyPrinter();
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final SerializationConfig _config;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final SerializerProvider _provider;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final SerializerFactory _serializerFactory;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final JsonFactory _jsonFactory;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final JavaType _rootType;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final PrettyPrinter _prettyPrinter;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final FormatSchema _schema;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected ObjectWriter(ObjectMapper mapper, SerializationConfig config, JavaType rootType, PrettyPrinter pp)
/*     */   {
/*  94 */     this._config = config;
/*     */     
/*  96 */     this._provider = mapper._serializerProvider;
/*  97 */     this._serializerFactory = mapper._serializerFactory;
/*  98 */     this._jsonFactory = mapper._jsonFactory;
/*     */     
/* 100 */     this._rootType = rootType;
/* 101 */     this._prettyPrinter = pp;
/* 102 */     this._schema = null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected ObjectWriter(ObjectMapper mapper, SerializationConfig config)
/*     */   {
/* 112 */     this._config = config;
/*     */     
/* 114 */     this._provider = mapper._serializerProvider;
/* 115 */     this._serializerFactory = mapper._serializerFactory;
/* 116 */     this._jsonFactory = mapper._jsonFactory;
/*     */     
/* 118 */     this._rootType = null;
/* 119 */     this._prettyPrinter = null;
/* 120 */     this._schema = null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected ObjectWriter(ObjectMapper mapper, SerializationConfig config, FormatSchema s)
/*     */   {
/* 131 */     this._config = config;
/*     */     
/* 133 */     this._provider = mapper._serializerProvider;
/* 134 */     this._serializerFactory = mapper._serializerFactory;
/* 135 */     this._jsonFactory = mapper._jsonFactory;
/*     */     
/* 137 */     this._rootType = null;
/* 138 */     this._prettyPrinter = null;
/* 139 */     this._schema = s;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected ObjectWriter(ObjectWriter base, SerializationConfig config, JavaType rootType, PrettyPrinter pp, FormatSchema s)
/*     */   {
/* 148 */     this._config = config;
/*     */     
/* 150 */     this._provider = base._provider;
/* 151 */     this._serializerFactory = base._serializerFactory;
/* 152 */     this._jsonFactory = base._jsonFactory;
/*     */     
/* 154 */     this._rootType = rootType;
/* 155 */     this._prettyPrinter = pp;
/* 156 */     this._schema = s;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected ObjectWriter(ObjectWriter base, SerializationConfig config)
/*     */   {
/* 166 */     this._config = config;
/*     */     
/* 168 */     this._provider = base._provider;
/* 169 */     this._serializerFactory = base._serializerFactory;
/* 170 */     this._jsonFactory = base._jsonFactory;
/* 171 */     this._schema = base._schema;
/*     */     
/* 173 */     this._rootType = base._rootType;
/* 174 */     this._prettyPrinter = base._prettyPrinter;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Version version()
/*     */   {
/* 184 */     return VersionUtil.versionFor(getClass());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public ObjectWriter withView(Class<?> view)
/*     */   {
/* 200 */     if (view == this._config.getSerializationView()) return this;
/* 201 */     return new ObjectWriter(this, this._config.withView(view));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public ObjectWriter withType(JavaType rootType)
/*     */   {
/* 211 */     if (rootType == this._rootType) { return this;
/*     */     }
/* 213 */     return new ObjectWriter(this, this._config, rootType, this._prettyPrinter, this._schema);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public ObjectWriter withType(Class<?> rootType)
/*     */   {
/* 223 */     return withType(this._config.constructType(rootType));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public ObjectWriter withType(TypeReference<?> rootType)
/*     */   {
/* 231 */     return withType(this._config.getTypeFactory().constructType(rootType.getType()));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public ObjectWriter withPrettyPrinter(PrettyPrinter pp)
/*     */   {
/* 242 */     if (pp == this._prettyPrinter) {
/* 243 */       return this;
/*     */     }
/*     */     
/* 246 */     if (pp == null) {
/* 247 */       pp = NULL_PRETTY_PRINTER;
/*     */     }
/* 249 */     return new ObjectWriter(this, this._config, this._rootType, pp, this._schema);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public ObjectWriter withDefaultPrettyPrinter()
/*     */   {
/* 260 */     return withPrettyPrinter(new DefaultPrettyPrinter());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public ObjectWriter withFilters(FilterProvider filterProvider)
/*     */   {
/* 271 */     if (filterProvider == this._config.getFilterProvider()) {
/* 272 */       return this;
/*     */     }
/* 274 */     return new ObjectWriter(this, this._config.withFilters(filterProvider));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public ObjectWriter withSchema(FormatSchema schema)
/*     */   {
/* 282 */     if (this._schema == schema) {
/* 283 */       return this;
/*     */     }
/* 285 */     return new ObjectWriter(this, this._config, this._rootType, this._prettyPrinter, schema);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void writeValue(JsonGenerator jgen, Object value)
/*     */     throws IOException, JsonGenerationException, JsonMappingException
/*     */   {
/* 301 */     if ((this._config.isEnabled(SerializationConfig.Feature.CLOSE_CLOSEABLE)) && ((value instanceof Closeable))) {
/* 302 */       _writeCloseableValue(jgen, value, this._config);
/*     */     } else {
/* 304 */       if (this._rootType == null) {
/* 305 */         this._provider.serializeValue(this._config, jgen, value, this._serializerFactory);
/*     */       } else {
/* 307 */         this._provider.serializeValue(this._config, jgen, value, this._rootType, this._serializerFactory);
/*     */       }
/* 309 */       if (this._config.isEnabled(SerializationConfig.Feature.FLUSH_AFTER_WRITE_VALUE)) {
/* 310 */         jgen.flush();
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void writeValue(File resultFile, Object value)
/*     */     throws IOException, JsonGenerationException, JsonMappingException
/*     */   {
/* 328 */     _configAndWriteValue(this._jsonFactory.createJsonGenerator(resultFile, JsonEncoding.UTF8), value);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void writeValue(OutputStream out, Object value)
/*     */     throws IOException, JsonGenerationException, JsonMappingException
/*     */   {
/* 345 */     _configAndWriteValue(this._jsonFactory.createJsonGenerator(out, JsonEncoding.UTF8), value);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void writeValue(Writer w, Object value)
/*     */     throws IOException, JsonGenerationException, JsonMappingException
/*     */   {
/* 361 */     _configAndWriteValue(this._jsonFactory.createJsonGenerator(w), value);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public String writeValueAsString(Object value)
/*     */     throws IOException, JsonGenerationException, JsonMappingException
/*     */   {
/* 374 */     SegmentedStringWriter sw = new SegmentedStringWriter(this._jsonFactory._getBufferRecycler());
/* 375 */     _configAndWriteValue(this._jsonFactory.createJsonGenerator(sw), value);
/* 376 */     return sw.getAndClear();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public byte[] writeValueAsBytes(Object value)
/*     */     throws IOException, JsonGenerationException, JsonMappingException
/*     */   {
/* 389 */     ByteArrayBuilder bb = new ByteArrayBuilder(this._jsonFactory._getBufferRecycler());
/* 390 */     _configAndWriteValue(this._jsonFactory.createJsonGenerator(bb, JsonEncoding.UTF8), value);
/* 391 */     byte[] result = bb.toByteArray();
/* 392 */     bb.release();
/* 393 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public boolean canSerialize(Class<?> type)
/*     */   {
/* 404 */     return this._provider.hasSerializerFor(this._config, type, this._serializerFactory);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final void _configAndWriteValue(JsonGenerator jgen, Object value)
/*     */     throws IOException, JsonGenerationException, JsonMappingException
/*     */   {
/* 420 */     if (this._prettyPrinter != null) {
/* 421 */       PrettyPrinter pp = this._prettyPrinter;
/* 422 */       jgen.setPrettyPrinter(pp == NULL_PRETTY_PRINTER ? null : pp);
/* 423 */     } else if (this._config.isEnabled(SerializationConfig.Feature.INDENT_OUTPUT)) {
/* 424 */       jgen.useDefaultPrettyPrinter();
/*     */     }
/*     */     
/* 427 */     if (this._schema != null) {
/* 428 */       jgen.setSchema(this._schema);
/*     */     }
/*     */     
/* 431 */     if ((this._config.isEnabled(SerializationConfig.Feature.CLOSE_CLOSEABLE)) && ((value instanceof Closeable))) {
/* 432 */       _configAndWriteCloseable(jgen, value, this._config);
/* 433 */       return;
/*     */     }
/* 435 */     boolean closed = false;
/*     */     try {
/* 437 */       if (this._rootType == null) {
/* 438 */         this._provider.serializeValue(this._config, jgen, value, this._serializerFactory);
/*     */       } else {
/* 440 */         this._provider.serializeValue(this._config, jgen, value, this._rootType, this._serializerFactory);
/*     */       }
/* 442 */       closed = true;
/* 443 */       jgen.close(); return;
/*     */ 
/*     */     }
/*     */     finally
/*     */     {
/* 448 */       if (!closed) {
/*     */         try {
/* 450 */           jgen.close();
/*     */         }
/*     */         catch (IOException ioe) {}
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private final void _configAndWriteCloseable(JsonGenerator jgen, Object value, SerializationConfig cfg)
/*     */     throws IOException, JsonGenerationException, JsonMappingException
/*     */   {
/* 463 */     Closeable toClose = (Closeable)value;
/*     */     try {
/* 465 */       if (this._rootType == null) {
/* 466 */         this._provider.serializeValue(cfg, jgen, value, this._serializerFactory);
/*     */       } else {
/* 468 */         this._provider.serializeValue(cfg, jgen, value, this._rootType, this._serializerFactory);
/*     */       }
/*     */       
/* 471 */       if (this._schema != null) {
/* 472 */         jgen.setSchema(this._schema);
/*     */       }
/* 474 */       JsonGenerator tmpJgen = jgen;
/* 475 */       jgen = null;
/* 476 */       tmpJgen.close();
/* 477 */       Closeable tmpToClose = toClose;
/* 478 */       toClose = null;
/* 479 */       tmpToClose.close(); return;
/*     */ 
/*     */     }
/*     */     finally
/*     */     {
/* 484 */       if (jgen != null) {
/*     */         try {
/* 486 */           jgen.close();
/*     */         } catch (IOException ioe) {}
/*     */       }
/* 489 */       if (toClose != null) {
/*     */         try {
/* 491 */           toClose.close();
/*     */         }
/*     */         catch (IOException ioe) {}
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private final void _writeCloseableValue(JsonGenerator jgen, Object value, SerializationConfig cfg)
/*     */     throws IOException, JsonGenerationException, JsonMappingException
/*     */   {
/* 504 */     Closeable toClose = (Closeable)value;
/*     */     try {
/* 506 */       if (this._rootType == null) {
/* 507 */         this._provider.serializeValue(cfg, jgen, value, this._serializerFactory);
/*     */       } else {
/* 509 */         this._provider.serializeValue(cfg, jgen, value, this._rootType, this._serializerFactory);
/*     */       }
/* 511 */       if (this._config.isEnabled(SerializationConfig.Feature.FLUSH_AFTER_WRITE_VALUE)) {
/* 512 */         jgen.flush();
/*     */       }
/* 514 */       Closeable tmpToClose = toClose;
/* 515 */       toClose = null;
/* 516 */       tmpToClose.close(); return;
/*     */     } finally {
/* 518 */       if (toClose != null) {
/*     */         try {
/* 520 */           toClose.close();
/*     */         }
/*     */         catch (IOException ioe) {}
/*     */       }
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\ObjectWriter.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */