/*     */ package org.codehaus.jackson.map.deser;
/*     */ 
/*     */ import java.io.DataInputStream;
/*     */ import java.io.IOException;
/*     */ import java.net.InetAddress;
/*     */ import java.net.URI;
/*     */ import java.net.URL;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Currency;
/*     */ import java.util.Locale;
/*     */ import java.util.TimeZone;
/*     */ import java.util.UUID;
/*     */ import java.util.regex.Pattern;
/*     */ import org.codehaus.jackson.JsonParser;
/*     */ import org.codehaus.jackson.JsonProcessingException;
/*     */ import org.codehaus.jackson.JsonToken;
/*     */ import org.codehaus.jackson.map.DeserializationContext;
/*     */ 
/*     */ public abstract class FromStringDeserializer<T> extends StdScalarDeserializer<T>
/*     */ {
/*     */   protected FromStringDeserializer(Class<?> vc)
/*     */   {
/*  23 */     super(vc);
/*     */   }
/*     */   
/*     */   public static Iterable<FromStringDeserializer<?>> all()
/*     */   {
/*  28 */     ArrayList<FromStringDeserializer<?>> all = new ArrayList();
/*     */     
/*  30 */     all.add(new UUIDDeserializer());
/*  31 */     all.add(new URLDeserializer());
/*  32 */     all.add(new URIDeserializer());
/*  33 */     all.add(new CurrencyDeserializer());
/*  34 */     all.add(new PatternDeserializer());
/*     */     
/*  36 */     all.add(new LocaleDeserializer());
/*     */     
/*  38 */     all.add(new InetAddressDeserializer());
/*  39 */     all.add(new TimeZoneDeserializer());
/*     */     
/*  41 */     return all;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public final T deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/*  49 */     if (jp.getCurrentToken() == JsonToken.VALUE_STRING) {
/*  50 */       String text = jp.getText().trim();
/*     */       
/*  52 */       if (text.length() == 0) {
/*  53 */         return null;
/*     */       }
/*     */       try {
/*  56 */         T result = _deserialize(text, ctxt);
/*  57 */         if (result != null) {
/*  58 */           return result;
/*     */         }
/*     */       }
/*     */       catch (IllegalArgumentException iae) {}
/*     */       
/*  63 */       throw ctxt.weirdStringException(this._valueClass, "not a valid textual representation");
/*     */     }
/*  65 */     if (jp.getCurrentToken() == JsonToken.VALUE_EMBEDDED_OBJECT)
/*     */     {
/*  67 */       Object ob = jp.getEmbeddedObject();
/*  68 */       if (ob == null) {
/*  69 */         return null;
/*     */       }
/*  71 */       if (this._valueClass.isAssignableFrom(ob.getClass())) {
/*  72 */         return (T)ob;
/*     */       }
/*  74 */       return (T)_deserializeEmbedded(ob, ctxt);
/*     */     }
/*  76 */     throw ctxt.mappingException(this._valueClass);
/*     */   }
/*     */   
/*     */ 
/*     */   protected abstract T _deserialize(String paramString, DeserializationContext paramDeserializationContext)
/*     */     throws IOException, JsonProcessingException;
/*     */   
/*     */   protected T _deserializeEmbedded(Object ob, DeserializationContext ctxt)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/*  86 */     throw ctxt.mappingException("Don't know how to convert embedded Object of type " + ob.getClass().getName() + " into " + this._valueClass.getName());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static class UUIDDeserializer
/*     */     extends FromStringDeserializer<UUID>
/*     */   {
/*     */     public UUIDDeserializer()
/*     */     {
/*  99 */       super();
/*     */     }
/*     */     
/*     */     protected UUID _deserialize(String value, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 105 */       return UUID.fromString(value);
/*     */     }
/*     */     
/*     */ 
/*     */     protected UUID _deserializeEmbedded(Object ob, DeserializationContext ctxt)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 112 */       if ((ob instanceof byte[])) {
/* 113 */         byte[] bytes = (byte[])ob;
/* 114 */         if (bytes.length != 16) {
/* 115 */           ctxt.mappingException("Can only construct UUIDs from 16 byte arrays; got " + bytes.length + " bytes");
/*     */         }
/*     */         
/* 118 */         DataInputStream in = new DataInputStream(new java.io.ByteArrayInputStream(bytes));
/* 119 */         long l1 = in.readLong();
/* 120 */         long l2 = in.readLong();
/* 121 */         return new UUID(l1, l2);
/*     */       }
/* 123 */       super._deserializeEmbedded(ob, ctxt);
/* 124 */       return null;
/*     */     }
/*     */   }
/*     */   
/*     */   public static class URLDeserializer extends FromStringDeserializer<URL>
/*     */   {
/*     */     public URLDeserializer() {
/* 131 */       super();
/*     */     }
/*     */     
/*     */     protected URL _deserialize(String value, DeserializationContext ctxt)
/*     */       throws IOException
/*     */     {
/* 137 */       return new URL(value);
/*     */     }
/*     */   }
/*     */   
/*     */   public static class URIDeserializer extends FromStringDeserializer<URI>
/*     */   {
/*     */     public URIDeserializer() {
/* 144 */       super();
/*     */     }
/*     */     
/*     */     protected URI _deserialize(String value, DeserializationContext ctxt)
/*     */       throws IllegalArgumentException
/*     */     {
/* 150 */       return URI.create(value);
/*     */     }
/*     */   }
/*     */   
/*     */   public static class CurrencyDeserializer extends FromStringDeserializer<Currency>
/*     */   {
/*     */     public CurrencyDeserializer() {
/* 157 */       super();
/*     */     }
/*     */     
/*     */ 
/*     */     protected Currency _deserialize(String value, DeserializationContext ctxt)
/*     */       throws IllegalArgumentException
/*     */     {
/* 164 */       return Currency.getInstance(value);
/*     */     }
/*     */   }
/*     */   
/*     */   public static class PatternDeserializer extends FromStringDeserializer<Pattern>
/*     */   {
/*     */     public PatternDeserializer() {
/* 171 */       super();
/*     */     }
/*     */     
/*     */ 
/*     */     protected Pattern _deserialize(String value, DeserializationContext ctxt)
/*     */       throws IllegalArgumentException
/*     */     {
/* 178 */       return Pattern.compile(value);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected static class LocaleDeserializer
/*     */     extends FromStringDeserializer<Locale>
/*     */   {
/*     */     public LocaleDeserializer()
/*     */     {
/* 190 */       super();
/*     */     }
/*     */     
/*     */     protected Locale _deserialize(String value, DeserializationContext ctxt)
/*     */       throws IOException
/*     */     {
/* 196 */       int ix = value.indexOf('_');
/* 197 */       if (ix < 0) {
/* 198 */         return new Locale(value);
/*     */       }
/* 200 */       String first = value.substring(0, ix);
/* 201 */       value = value.substring(ix + 1);
/* 202 */       ix = value.indexOf('_');
/* 203 */       if (ix < 0) {
/* 204 */         return new Locale(first, value);
/*     */       }
/* 206 */       String second = value.substring(0, ix);
/* 207 */       return new Locale(first, second, value.substring(ix + 1));
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected static class InetAddressDeserializer
/*     */     extends FromStringDeserializer<InetAddress>
/*     */   {
/*     */     public InetAddressDeserializer()
/*     */     {
/* 219 */       super();
/*     */     }
/*     */     
/*     */     protected InetAddress _deserialize(String value, DeserializationContext ctxt)
/*     */       throws IOException
/*     */     {
/* 225 */       return InetAddress.getByName(value);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected static class TimeZoneDeserializer
/*     */     extends FromStringDeserializer<TimeZone>
/*     */   {
/*     */     public TimeZoneDeserializer()
/*     */     {
/* 237 */       super();
/*     */     }
/*     */     
/*     */     protected TimeZone _deserialize(String value, DeserializationContext ctxt)
/*     */       throws IOException
/*     */     {
/* 243 */       return TimeZone.getTimeZone(value);
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\deser\FromStringDeserializer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */