package prefuse.data.io.sql;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.logging.Logger;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.data.io.DataIOException;
import prefuse.data.util.Index;

public class DatabaseDataSource
{
  private static final Logger s_logger = Logger.getLogger(DatabaseDataSource.class.getName());
  protected Connection m_conn;
  protected Statement m_stmt;
  protected SQLDataHandler m_handler;
  
  DatabaseDataSource(Connection paramConnection, SQLDataHandler paramSQLDataHandler)
  {
    this.m_conn = paramConnection;
    this.m_handler = paramSQLDataHandler;
  }
  
  public synchronized Table getData(String paramString)
    throws DataIOException
  {
    return getData(null, paramString, null);
  }
  
  public synchronized Table getData(String paramString1, String paramString2)
    throws DataIOException
  {
    return getData(null, paramString1, paramString2);
  }
  
  public synchronized Table getData(Table paramTable, String paramString)
    throws DataIOException
  {
    return getData(paramTable, paramString, null);
  }
  
  public synchronized Table getData(Table paramTable, String paramString1, String paramString2)
    throws DataIOException
  {
    return getData(paramTable, paramString1, paramString2, null);
  }
  
  public synchronized Table getData(Table paramTable, String paramString1, String paramString2, Object paramObject)
    throws DataIOException
  {
    ResultSet localResultSet;
    try
    {
      localResultSet = executeQuery(paramString1);
    }
    catch (SQLException localSQLException)
    {
      throw new DataIOException(localSQLException);
    }
    return process(paramTable, localResultSet, paramString2, paramObject);
  }
  
  public void loadData(Table paramTable, String paramString)
  {
    loadData(paramTable, paramString, null, null, null);
  }
  
  public void loadData(Table paramTable, String paramString1, String paramString2)
  {
    loadData(paramTable, paramString1, paramString2, null, null);
  }
  
  public void loadData(Table paramTable, String paramString, Object paramObject)
  {
    loadData(paramTable, paramString, null, paramObject, null);
  }
  
  public void loadData(Table paramTable, String paramString1, String paramString2, Object paramObject)
  {
    loadData(paramTable, paramString1, paramString2, paramObject, null);
  }
  
  public void loadData(Table paramTable, String paramString1, String paramString2, Object paramObject, DataSourceWorker.Listener paramListener)
  {
    DataSourceWorker.Entry localEntry = new DataSourceWorker.Entry(this, paramTable, paramString1, paramString2, paramObject, paramListener);
    DataSourceWorker.submit(localEntry);
  }
  
  private ResultSet executeQuery(String paramString)
    throws SQLException
  {
    if (this.m_stmt == null) {
      this.m_stmt = this.m_conn.createStatement();
    }
    long l1 = System.currentTimeMillis();
    s_logger.info("Issuing query: " + paramString);
    ResultSet localResultSet = this.m_stmt.executeQuery(paramString);
    long l2 = System.currentTimeMillis() - l1;
    s_logger.info("External query processing completed: " + l2 / 1000L + "." + l2 % 1000L + " seconds.");
    return localResultSet;
  }
  
  protected Table process(Table paramTable, ResultSet paramResultSet, String paramString, Object paramObject)
    throws DataIOException
  {
    int i = 0;
    long l1 = System.currentTimeMillis();
    try
    {
      ResultSetMetaData localResultSetMetaData = paramResultSet.getMetaData();
      int j = localResultSetMetaData.getColumnCount();
      if (paramTable == null)
      {
        paramTable = getSchema(localResultSetMetaData, this.m_handler).instantiate();
        if (paramString != null) {
          try
          {
            paramTable.index(paramString);
            s_logger.info("Indexed field: " + paramString);
          }
          catch (Exception localException)
          {
            s_logger.warning("Error indexing field: " + paramString);
          }
        }
      }
      paramObject = paramObject == null ? paramTable : paramObject;
      while (paramResultSet.next())
      {
        synchronized (paramObject)
        {
          int k = getExistingRow(paramTable, paramResultSet, paramString);
          if (k < 0) {
            k = paramTable.addRow();
          }
          for (int m = 1; m <= j; m++) {
            this.m_handler.process(paramTable, k, paramResultSet, m);
          }
        }
        i++;
      }
    }
    catch (SQLException localSQLException)
    {
      throw new DataIOException(localSQLException);
    }
    long l2 = System.currentTimeMillis() - l1;
    s_logger.info("Internal query processing completed: " + i + " rows, " + l2 / 1000L + "." + l2 % 1000L + " seconds.");
    return paramTable;
  }
  
  protected int getExistingRow(Table paramTable, ResultSet paramResultSet, String paramString)
    throws SQLException
  {
    if (paramString == null) {
      return -1;
    }
    Class localClass = paramTable.getColumnType(paramString);
    if (localClass == null) {
      return -1;
    }
    Index localIndex = paramTable.index(paramString);
    if (localClass == Integer.TYPE) {
      return localIndex.get(paramResultSet.getInt(paramString));
    }
    if (localClass == Long.TYPE) {
      return localIndex.get(paramResultSet.getLong(paramString));
    }
    if (localClass == Float.TYPE) {
      return localIndex.get(paramResultSet.getFloat(paramString));
    }
    if (localClass == Double.TYPE) {
      return localIndex.get(paramResultSet.getDouble(paramString));
    }
    if (!localClass.isPrimitive()) {
      return localIndex.get(paramResultSet.getObject(paramString));
    }
    return -1;
  }
  
  public Schema getSchema(ResultSetMetaData paramResultSetMetaData, SQLDataHandler paramSQLDataHandler)
    throws SQLException
  {
    int i = paramResultSetMetaData.getColumnCount();
    Schema localSchema = new Schema(i);
    for (int j = 1; j <= i; j++)
    {
      String str = paramResultSetMetaData.getColumnName(j);
      int k = paramResultSetMetaData.getColumnType(j);
      Class localClass = paramSQLDataHandler.getDataType(str, k);
      if (localClass != null) {
        localSchema.addColumn(str, localClass);
      }
    }
    return localSchema;
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\io\sql\DatabaseDataSource.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */